@@ -28,8 +28,9 @@
 extern "C" {
 #endif
 
-#define GRN_PAT_MAX_KEY_SIZE       GRN_TABLE_MAX_KEY_SIZE
-#define GRN_PAT_MAX_TOTAL_KEY_SIZE (UINT32_MAX - 1)
+#define GRN_PAT_MAX_KEY_SIZE             GRN_TABLE_MAX_KEY_SIZE
+#define GRN_PAT_MAX_TOTAL_KEY_SIZE       (UINT32_MAX - 1)
+#define GRN_PAT_MAX_TOTAL_KEY_SIZE_LARGE (UINT64_MAX - 1)
 
 struct _grn_pat {
   grn_db_obj obj;
@@ -76,7 +77,8 @@ struct grn_pat_header {
   uint32_t truncated;
   uint32_t n_dirty_opens;
   uint64_t wal_id;
-  uint32_t reserved[1000];
+  uint64_t curr_key_large;
+  uint32_t reserved[998];
   grn_pat_delinfo delinfos[GRN_PAT_NDELINFOS];
   grn_id garbages[GRN_PAT_MAX_KEY_SIZE + 1];
 };
@@ -135,8 +137,10 @@ grn_pat_fuzzy_search(grn_ctx *ctx,
                      grn_fuzzy_search_optarg *args,
                      grn_hash *h);
 
-uint32_t
+uint64_t
 grn_pat_total_key_size(grn_ctx *ctx, grn_pat *pat);
+uint64_t
+grn_pat_max_total_key_size(grn_ctx *ctx, grn_pat *pat);
 
 bool
 grn_pat_is_key_encoded(grn_ctx *ctx, grn_pat *pat);
@@ -153,7 +157,7 @@ grn_rc
 grn_pat_wal_recover(grn_ctx *ctx, grn_pat *pat);
 grn_rc
 grn_pat_warm(grn_ctx *ctx, grn_pat *pat);
-int
+uint64_t
 grn_pat_defrag(grn_ctx *ctx, grn_pat *pat);
 
 #ifdef __cplusplus
